@@ -1,51 +1,22 @@
 //! ## Programs
 //!
-//! Programs are composed of Lanes and Cards. A Card consumes inputs and produces outputs.
-//! Cards will always consume from the top of the stack downwards and push their outputs to
-//! the stack. This means that subprogram composition is not a commutative operation. (Consider
-//! Cards A, B and C. Then the composition ABC is not the same as BAC if A != B. )
-//!
-//! Execution will begin at the first `Lane`.
-//!
-//! Example Program serialized as JSON
-//! ```
-//! const PROGRAM: &str = r#"{
-//!   "lanes":[
-//!      {
-//!         "name":"Main",
-//!         "cards":[
-//!            {
-//!               "ty": "ScalarInt",
-//!               "val":42
-//!            },
-//!            {
-//!               "ty": "CallNative", "val":"log_scalar"
-//!            }
-//!         ]
-//!      }
-//!   ]
-//!}"#;
-//!
-//!let compilation_unit = serde_json::from_str(PROGRAM).unwrap();
-//!cao_lang::compiler::compile(&compilation_unit, None).unwrap();
-//!```
+//! __TBA__
 //!
 
 #![recursion_limit = "256"]
 
 mod alloc;
 pub mod collections;
+pub mod compiled_program;
 pub mod compiler;
 pub mod instruction;
 pub mod prelude;
 pub mod procedures;
-pub mod program;
 pub mod traits;
 pub mod value;
 pub mod vm;
 
 mod bytecode;
-mod macros;
 
 pub mod version {
     include!(concat!(env!("OUT_DIR"), "/cao_lang_version.rs"));
@@ -129,20 +100,20 @@ pub type VarName = ArrayString<64>;
 /// Subprograms consume their inputs and produce outputs.
 #[derive(Clone)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-pub struct SubProgram<'a> {
-    pub name: &'a str,
-    pub description: &'a str,
+pub struct SubProgram {
+    pub name: String,
+    pub description: String,
     pub ty: SubProgramType,
 
     /// Human readable descriptions of the output
-    pub output: Box<[&'a str]>,
+    pub output: Box<[String]>,
     /// Human readable descriptions of inputs
-    pub input: Box<[&'a str]>,
+    pub input: Box<[String]>,
     /// Human readable descriptions of properties
-    pub properties: Box<[&'a str]>,
+    pub properties: Box<[String]>,
 }
 
-impl<'a> std::fmt::Debug for SubProgram<'a> {
+impl std::fmt::Debug for SubProgram {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("SubProgram")
             .field("name", &self.name)
@@ -189,8 +160,8 @@ impl SubProgramType {
 macro_rules! subprogram_description {
     ($name: expr, $description: expr, $ty: expr, [$($inputs: expr),*], [$($outputs: expr),*], [$($properties: expr),*]) => {
         SubProgram {
-            name: $name,
-            description: $description,
+            name: $name.to_string(),
+            description: $description.to_string(),
             ty: $ty,
             input: subprogram_description!(@input $($inputs),*) ,
             output: subprogram_description!(@input $($outputs),*),
@@ -201,7 +172,7 @@ macro_rules! subprogram_description {
     (@input $($lst: expr),*) => {
         vec![
             $(
-                $lst
+                $lst.to_string()
             ),*
         ]
         .into_boxed_slice()
